package org.netkernel.lang.groovy.endpoint;

import java.util.*;

import groovy.lang.GroovyClassLoader;
import groovy.lang.GroovyCodeSource;

import org.codehaus.groovy.control.CompilerConfiguration;
import org.netkernel.container.ISpaceListener;
import org.netkernel.container.impl.Kernel;
import org.netkernel.lang.groovy.representation.CompiledGroovyRep;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.layer1.representation.IDeterminateStringRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.ISpace;

public class GroovyCompiler extends StandardTransreptorImpl implements ISpaceListener
{
	private CompilerConfiguration mCompilerConfig;
	private Map<List<ISpace>,GroovyClassLoader> mCLCache=new HashMap();
	
	public GroovyCompiler()
	{	this.declareThreadSafe();
		this.declareToRepresentation(CompiledGroovyRep.class);
		CompilerConfiguration config=new CompilerConfiguration();
		config.setDebug(true);
	}

	@Override
	protected void postCommission(INKFRequestContext aContext) throws Exception
	{	Kernel k=(Kernel)aContext.getKernelContext().getKernel();
		k.addSpaceListener(this);
	}
	
	@Override
	protected void preDecommission(INKFRequestContext aContext) throws Exception
	{	Kernel k=(Kernel)aContext.getKernelContext().getKernel();
		k.removeSpaceListener(this);
	}
	
	public void spaceChanged()
	{	// dereference classloaders so that stale ones can be GCed
		synchronized(mCLCache)
		{	mCLCache.clear();
		}
	}

	@Override
	public void onTransrept(INKFRequestContext context) throws Exception
	{	
		IRequestScopeLevel scope=context.getKernelContext().getThisKernelRequest().getRequestScope();
		IRequestScopeLevel s2=scope;
		List<ISpace> l=new ArrayList(16);
		while(s2!=null)
		{	ISpace space=s2.getSpace();
			ClassLoader cl=space.getClassLoader();
			if (cl!=null)
			{	if (!l.contains(space))
				{	l.add(space);
				}
			}
			s2=s2.getParent();
		}
		GroovyClassLoader gcl=mCLCache.get(l);
		if (gcl==null)
		{	
			scope=RequestScopeLevelImpl.createRootScopeLevel(l.get(l.size()-1));
			for (int i=l.size()-2; i>=0; i--)
			{	scope=RequestScopeLevelImpl.createNonDurableScopeLevel(l.get(i), scope);
			}
			
			gcl=new GroovyClassLoader(new RequestScopeClassLoader(scope), mCompilerConfig);
			synchronized(mCLCache)
			{	if(mCLCache.size()>200)
				{	mCLCache.clear();			//Safety limit to ensure this can't leak but still gives typical optimal reuse of classloaders				
				}
				mCLCache.put(l,gcl);
			}
		}
		
		IDeterminateStringRepresentation script=context.sourcePrimary(IDeterminateStringRepresentation.class);
		Class c=null;
		synchronized(gcl)
		{	gcl.clearCache();
			c=gcl.parseClass(script.getString());
		}
		context.createResponseFrom(new CompiledGroovyRep(c, script.getString(), context.getKernelContext().getKernel().inDebug()));
	}
}
