package org.netkernel.lang.groovy.endpoint;

import java.util.List;

import groovy.lang.Binding;
import groovy.lang.Script;

import org.codehaus.groovy.runtime.InvokerHelper;
import org.netkernel.container.IKernel;
import org.netkernel.lang.groovy.representation.CompiledGroovyRep;
import org.netkernel.layer0.meta.IEndpointArgumentMeta;
import org.netkernel.layer0.meta.IPrototypeParameterMeta;
import org.netkernel.layer0.meta.impl.EndpointArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFParameters;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;


public class GroovyPrototype extends StandardAccessorImpl
{	
	private int mVerbs;
	
	public GroovyPrototype()
	{	declareThreadSafe();
		declareTolerantParameters();
		
		//IPrototypeParameterMeta[] params = mergeParameterArray(StandardAccessorImpl.sPrototypeParameters, sPrototypeParameters);
		//declareParameters(params);
	}
	
	public String toString()
	{	return mMeta.getName();
	}
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aContext) throws Exception
	{	INKFParameters params = this.getParameters();
	
		Object script = getParameter("script");
		if (script instanceof IIdentifier)
		{	declareName(((IIdentifier)script).toString());
		}
		else
		{	declareName("Groovy");
		}
	
		String verbsString = (String)params.getParamValue("verbs");
		mVerbs = Utils.parseVerbList(verbsString);
		if (mVerbs!=0)
		{	declareSupportedVerbs(mVerbs);
		}
		
		//setup from representations
		List<Object> paramValues=params.getParamValues("fromRepresentation");
		Class[] fromRepresentations = new Class[paramValues.size()];
		for (int i=0; i<fromRepresentations.length; i++)
		{	String fromRepString=(String)paramValues.get(i);
			Class fromRep=aContext.getClassLoader().loadClass(fromRepString);
			fromRepresentations[i]=fromRep;
		}
		IEndpointArgumentMeta arg= new EndpointArgumentMetaImpl(IEndpointArgumentMeta.PRIMARY_PART,null,null,null,null,false,fromRepresentations);
		declareArgument(arg);
		
		//setup to representations
		 paramValues=params.getParamValues("toRepresentation");
		for (int i=0; i<paramValues.size(); i++)
		{	String toRepString=(String)paramValues.get(i);
			Class toRep=aContext.getClassLoader().loadClass(toRepString);
			declareSourceRepresentation(toRep);
		}
		
		
		super.onCommissionEndpoint(aKernel, aContext);
	}
	
	public void onRequest(INKFRequestContext context) throws Exception
	{	
		if (context.getThisRequest().getVerb()==INKFRequestReadOnly.VERB_META)
		{	context.createResponseFrom(mMeta);
		}
		else
		{	CompiledGroovyRep opt=(CompiledGroovyRep)context.source("param:script", CompiledGroovyRep.class);
			ClassLoader cl=new RequestScopeClassLoader(context.getKernelContext().getRequestScope());
			Thread.currentThread().setContextClassLoader(cl);
			Binding b=new Binding();
			b.setVariable("context", context);
			InvokerHelper.createScript(opt.getCompiledScript(), b).run();
		}
	}	
}