package org.netkernel.lang.groovy.endpoint;

import groovy.lang.GString;

import org.codehaus.groovy.runtime.GStringImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer1.representation.DeterminateStringRep;
import org.netkernel.layer1.representation.IDeterminateStringRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;

public class GroovyStringTransreptor extends StandardTransreptorImpl
{

	public GroovyStringTransreptor()
	{
		this.declareFromRepresentation(GStringImpl.class);
		this.declareToRepresentation(String.class);
		this.declareToRepresentation(IReadableBinaryStreamRepresentation.class);
		this.declareToRepresentation(IDeterminateStringRepresentation.class);
		this.declareThreadSafe();
	}
	
	@Override
	public void onTransrept(INKFRequestContext context) throws Exception
	{
		INKFResponseReadOnly responseIn = context.sourcePrimaryForResponse(GStringImpl.class);
		GString gs =(GString)responseIn.getRepresentation();
		Class to=context.getThisRequest().getRepresentationClass();
		Object rep=null;
		if(to.isAssignableFrom(String.class))
		{
			rep=gs.toString();
		}
		else if(to.isAssignableFrom(IDeterminateStringRepresentation.class))
		{	
			rep=new DeterminateStringRep(gs.toString());			
		}
		else
		{
			rep = new ByteArrayRepresentation(gs.toString().getBytes("UTF-8"), "UTF-8");
		}
		INKFResponse responseOut = context.createResponseFrom(rep);
		String mime=responseIn.getMimeType();
		if (mime.equals(INKFResponseReadOnly.MIME_UNKNOWN))
		{	mime="text/plain";
		}
		responseOut.setMimeType(mime);
	}
	
}
